<?php

namespace Bazookas\AdminBundle\Controller\AdminController;

use Bazookas\AdminBundle\Controller\Base\BaseAdminListController;
use Bazookas\AdminBundle\PageBuilder\CopyPageBuilder;
use Bazookas\AdminBundle\PageBuilder\Interfaces\ListPageBuilderInterface;
use Bazookas\CommonBundle\Security\Roles;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class CopyAdminController extends BaseAdminListController
{

  public function __construct() {
    parent::__construct();

    $this->builders = [
      self::ACTION_LIST => '@bazookas.admin.pagebuilders.CopyPageBuilder'
    ];

    //Define ajax builders
    $this->builders[self::AJAX_LIST] = $this->builders[self::ACTION_LIST];
  }

  /**
   * @inheritDoc
   */
  protected function getEntityClass()
  {
    //We're not using an entity for this so return null
    return null;
  }

  /**
   * @inheritDoc
   */
  protected function hasAccess($action)
  {
    return $this->isGranted(Roles::ROLE_COPY_MANAGER);
  }

  /**
   * @inheritDoc
   */
  protected function getFormClass($action)
  {
    //TODO: implement this
    return null;
  }

  /**
   * @param Request $request
   * @param ListPageBuilderInterface $builder
   * @return ListPageBuilderInterface
   */
  protected function modifyListBuilder(Request $request, ListPageBuilderInterface $builder) {
    /**
     * @var CopyPageBuilder $builder
     */
    $builder
//      ->setAjaxPageable(true)
      ->setInlineEditable(true)
    ;

    if ($this->isGranted(Roles::ROLE_SUPER_ADMIN)) {
      $builder
        ->setTranslationDomains(['admin', 'messages'])
        ->addChoiceFilterField('domain', [
          'choices' => [
            'admin' => 'admin',
            'messages' => 'messages'
          ]
        ])
      ;
    }

    return $builder;
  }

  protected function ajaxInlineEditAction(Request $request, $action) {
    //Get the page builder for the current action
    $builder = $this->getPageBuilder(self::ACTION_LIST);
    $builder = $this->modifyBuilder($request, self::ACTION_LIST, $builder);

    $translationDomains = $builder->getTranslationDomains();
    $translationService = $this->get('bazookas.admin.service.TranslationDataService');

    $data = json_decode($request->getContent(), true);

    $translationService->updateTranslation($data, $translationDomains);

    return new JsonResponse();
  }
}
