<?php

namespace Bazookas\AdminBundle\EventListener\Entity;

use Bazookas\AdminBundle\Entity\Interfaces\AuditableInterface;
use Bazookas\AdminBundle\Entity\Interfaces\AuditLogInterface;
use Bazookas\AdminBundle\Entity\Interfaces\Versionable;
use Doctrine\Common\Cache\Version;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\OnFlushEventArgs;
use Doctrine\ORM\UnitOfWork;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorage;

class VersioningListener {


  private $tokenStorage;

  public function __construct(TokenStorage $tokenStorage) {
    $this->tokenStorage = $tokenStorage;
  }

  public function prePersist(LifecycleEventArgs $args) {
    $object = $args->getObject();
    if ($object instanceof Versionable) {
      $token = $this->tokenStorage->getToken();
      $user = null;
      if (!empty($token)) {
        $user = $token->getUser();
      }

      $object->setCreator($user);
    }
  }

  public function onFlush(OnFlushEventArgs $args) {
    $em = $args->getEntityManager();
    $uow = $em->getUnitOfWork();

    foreach ($uow->getScheduledEntityUpdates() as $entity) {
      if ($entity instanceof Versionable) {
        $changeSet = $uow->getEntityChangeSet($entity);
        $newVersion = $this->createNewVersion($changeSet, $entity);

        $em->persist($newVersion);
        $uow->computeChangeSet($em->getClassMetadata(get_class($newVersion)), $newVersion);
        $uow->computeChangeSet($em->getClassMetadata(get_class($entity)), $entity);
      }
    }
  }

  private function createNewVersion(array $changeSet, Versionable $entity) {
    $token = $this->tokenStorage->getToken();
    $user = null;
    if (!empty($token)) {
      $user = $token->getUser();
    }

    $newVersion = clone $entity;
    $newVersion
      ->setCreated(new \DateTime('now', new \DateTimeZone('UTC')))
      ->setId(null)
      ->setChildren([])
      ->setCreator($user)
      ->setParent($entity)
    ;

    //Fill the entity with the old values again
    foreach($changeSet as $field => $data) {
      $entity->{'set'.ucfirst($field)}($data[0]);
    }

    $entity
      ->addChild($newVersion)
    ;

    return $newVersion;
  }
}
