<?php

/*
 * This file is part of the FOSUserBundle package.
 *
 * (c) FriendsOfSymfony <http://friendsofsymfony.github.com/>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Bazookas\AdminBundle\Form;

use FOS\UserBundle\Util\LegacyFormHelper;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Security\Core\Validator\Constraints\UserPassword;
use Symfony\Component\Form\Extension\Core\Type as Type;

class ProfileFormType extends AbstractType
{
  private $class;

  /**
   * @param string $class The User class name
   */
  public function __construct($class = 'FOS\UserBundle\Model\User')
  {
    $this->class = $class;
  }

  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $this->buildUserForm($builder, $options);

    $builder->add('current_password', Type\PasswordType::class, [
      'label' => 'admin.profile.edit.form.password',
      'mapped' => false,
      'constraints' => new UserPassword(),
    ]);
  }

  /**
   * Builds the embedded form representing the user.
   *
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  protected function buildUserForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('username', null, ['label' => 'admin.profile.edit.form.username'])
      ->add('name', null, ['label' => 'admin.profile.edit.form.name'])
      ->add('surname', null, ['label' => 'admin.profile.edit.form.surname'])
      ->add('email', Type\EmailType::class, ['label' => 'admin.profile.edit.form.email']);
  }

  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => $this->class,
      'csrf_token_id' => 'profile',
      'translation_domain' => 'admin',
    ]);
  }

  public function getBlockPrefix()
  {
    return 'fos_user_profile';
  }
}
