<?php

namespace Bazookas\AdminBundle\PageBuilder\Base;

use Bazookas\AdminBundle\AdminElements\Elements\FormPanelElement;
use Bazookas\AdminBundle\AdminElements\Elements\HeaderElement;
use Bazookas\AdminBundle\PageBuilder\Interfaces\FormPageBuilderInterface;
use Bazookas\AdminBundle\Repository\Interfaces\ListRepositoryInterface;
use Bazookas\CommonBundle\Entity\Interfaces\EntityInterface;
use Symfony\Component\Form\Form;
use Symfony\Component\HttpFoundation\Request;

class BaseFormPageBuilder extends BasePageBuilder implements FormPageBuilderInterface
{

  /**
   * @var Form
   */
  protected $form;

  /**
   * @var array
   */
  protected $formOptions = [];

  /**
   * @var Mixed
   */
  protected $entity;

  /**
   * @var mixed
   */
  protected $entityId;

  /**
   * @var string
   */
  protected $headerLabel;

  /**
   * @return Form
   */
  public function getForm()
  {
    return $this->form;
  }

  /**
   * @param Form $form
   * @return $this
   */
  public function setForm(Form $form)
  {
    $this->form = $form;
    return $this;
  }

  /**
   * @return array
   */
  public function getFormOptions(): array
  {
    return $this->formOptions;
  }

  /**
   * @param array $formOptions
   * @return $this
   */
  public function setFormOptions(array $formOptions)
  {
    $this->formOptions = $formOptions;

    return $this;
  }

  /**
   * @return mixed
   */
  public function getEntity()
  {
    if (empty($this->entity) && !empty($this->entityId)) {
      $entityRepo = $this->entityManager->getRepository($this->entityClass);
      $entity = $entityRepo->findOneById($this->entityId);
      $this->entity = $entity;
    } else {
      $this->entity = new $this->entityClass();
    }

    return $this->entity;
  }

  /**
   * @param EntityInterface $entity
   * @return $this
   */
  public function setEntity(EntityInterface $entity)
  {
    $this->entity = $entity;

    return $this;
  }

  /**
   * @return mixed
   */
  public function getEntityId()
  {
    return $this->entityId;
  }

  /**
   * @param mixed $entityId
   * @return $this
   */
  public function setEntityId($entityId)
  {
    $this->entityId = $entityId;

    return $this;
  }

  /**
   * @return string
   */
  public function getHeaderLabel()
  {
    return $this->headerLabel;
  }

  /**
   * @param $headerLabel
   * @return $this
   */
  public function setHeaderLabel($headerLabel)
  {
    $this->headerLabel = $headerLabel;

    return $this;
  }

  /**
   * @param Request $request
   * @return bool
   * @throws \Exception
   */
  public function handleForm(Request $request) {
    $form = $this->getForm();
    if (empty($form)) {
      throw new \Exception('The form is empty in ' . get_class($this) . ' handleForm' );
    }

    $form->handleRequest($request);

    if ($form->isSubmitted() && $form->isValid()) {
      $entityRepo = $this->entityManager->getRepository($this->entityClass);
      if (!$entityRepo instanceof ListRepositoryInterface) {
        throw new \Exception('The entity repository must implement the interface: ' . ListRepositoryInterface::class);
      }

      $entityRepo->updateEntities([$form->getData()]);
      return true;
    } else {
      $this->form = $form;
      return false;
    }
  }

  /**
   * @inheritDoc
   */
  public function buildPage(Request $request) {
    $this->elements[] = new HeaderElement([], [
      'label' => $this->getHeaderLabel(),
      'class' => 'page-header text-overflow inline-block'
    ]);
    $this->elements[] = $this->createPageActionContainer();

    $form = $this->getForm();
    if (!empty($form)) {
      $this->elements[] = new FormPanelElement([], [
        'form' => $this->getForm()->createView()
      ]);
    }
  }

}
