<?php

namespace Bazookas\AdminBundle\Repository\Base;

use Bazookas\AdminBundle\Entity\Interfaces\Versionable;
use Bazookas\AdminBundle\Repository\Interfaces\ListRepositoryInterface;
use Bazookas\CommonBundle\Entity\Interfaces\Timestampable;
use \Bazookas\CommonBundle\Repository\Base\BaseRepository as CommonRepository;
use Doctrine\ORM\QueryBuilder;

abstract class BaseRepository extends CommonRepository implements ListRepositoryInterface
{
  /**
   * @return string
   */
  public function getEntityClass()
  {
    return $this->getClassMetadata()->getName();
  }

  /**
   * Create the list query builder, this function is used by the ListPageBuilder to create the admin list pages.
   * @param array $fields
   * @param array $params
   * @return QueryBuilder
   * @throws \Exception
   */
  public function createListQueryBuilder(array $fields, array $params) {
    //If we're dealing with a versionable entity use a different default query builder
    if (in_array(Versionable::class, class_implements($this->_entityName))) {
      //In order to be able to use the versionable behaviour the entity must implement the timestampable interface
      if (in_array(Timestampable::class, class_implements($this->_entityName))) {
        return $this->createGreatestRowPerGroupQueryBuilder($this->_entityName, 'entityId', 'created');
      } else {
        throw new \Exception('Your entity must implement the interface \''.Timestampable::class.'\' in order to be able to use the Versionable behaviour');
      }
    }

    //Default to a basic query builder
    return $this->createQueryBuilder('e');
  }

  /**
   * A hook to modify the list query builder before it is executed in the ListPageBuilder
   * @param QueryBuilder $qb
   * @param array $params
   * @return QueryBuilder
   */
  public function beforeListQueryExecution(QueryBuilder $qb, array $params) {
    return $qb;
  }

  protected function joinExsists(QueryBuilder $queryBuilder, $prefix, $alias)
  {
    $parts = $queryBuilder->getDQLPart('join');
    if (isset($parts[$prefix])) {
      foreach ($parts[$prefix] as $existingJoin) {
        if ($existingJoin->getAlias() === $alias) {
          return true;
        }
      }
    }

    return false;
  }


}
