## Admin lists

**NOTE**: DEPRECATED use [v2 admin controllers](/Resources/docs/Components/Admin.index.md)

This bundle contains a base controller class which can be extended in other bundles to easily create admin lists. The controller is located in `Bazookas\AdminBundle\Controller\ListBaseController`. Example usage of the controller can be seen in the `UserAdminController`.

The controller contains a config property holding a helper class for configuring the list.

The `ListBaseController` fires some events to make it easier for bundles to customize output. The following events are dispatched:
- `AdminListEvent::BEFORE_LIST_QUERY`: before the actual "list" query is executed. This way a bundle can listen to the event and filter the results according to the logged in user. The event contains a `queryBuilder` property which can be altered as you wish.
- `AdminListEvent::BEFORE_EXPORT_QUERY`: same as "list", but for exports.
- `AdminListEvent::AFTER_LIST_QUERY`: right after getting the list of entities. This can be used for altering the results.
- `AdminListEvent::BEFORE_EDIT`: before the edit form is rendered, use this to do some last access checks on entity.
- `AdminListEvent::BEFORE_REMOVE_ENTITY`: before removing the entity, the $event->entity property is set to allow for access checks on this entity, make sure to `throw new AccessDeniedException();` when user has no right to remove this entity
- `AdminListEvent::AFTER_REMOVE_ENTITY`: entity has been removed, you can now put flash messages or something
- `AdminListEvent::BEFORE_PERSIST`: before an entity is persisted
- `AdminListEvent::AFTER_PERSIST`: before an entity is persisted

Public event properties:
- `$event->queryBuilder`: set for `*_QUERY` events
- `$event->queryResult`: set for `AFTER_*_QUERY` events
- `$event->entity`: set for `EDIT`, `REMOVE` and `PERSIST` events
- `$event->action`: always set
- `$event->id`: set when ID is given to request (remove, edit) or when entity is persisted (add, edit)
- `$event->config`: always set, the config helper class for making final changes

#### Overrideable ListBaseController functions

Some functions in the list base are made only to override in an extending controller. These functions are executed at a specific time and provide an easy way to customize the controllers.

- `addListeners($dispatcher)`
  - executed before anything really happens
  - this way you can add listeners directly to the dispatcher for this specific controller
  - which means no "global" listeners must be added for a specific task
- `executeCustomActions($_action)`
  - is executed when no default actions are found
  - use to implement custom actions and benefit from the basic access checks
  - should return a valid response object
