## Installing
### With composer
Add the following to composer.json (composer will ask authentication for private bundles the first time you install a bazookas bundle)
```json
    "repositories": [
        {
            "type": "vcs",
            "url": "https://github.com/Bazookastudios/BazookasAdminBundle.git"
        }
    ],
```

And then: `composer require bazookas/admin-bundle`

### As a git submodule
Use this command in the root of your repository to add this bundle as a submodule.

```bash
git submodule add -b master --name AdminBundle https://github.com/Bazookastudios/BazookasAdminBundle.git ./src/Bazookas/AdminBundle
```
## Activating the installation
### Import in app/config.yml

```YAML
    - { resource: "@BazookasAdminBundle/Resources/config/config.yml" }
    - { resource: "@BazookasAdminBundle/Resources/config/services.yml" }
```

### Activate in AppKernel.php

```php
      new Bazookas\AdminBundle\BazookasAdminBundle(),
```

## Add to app/routing.yml

```YAML
# import FOS user routing, make sure this goes first so we can overwrite them
fos_user_security:
    resource: "@FOSUserBundle/Resources/config/routing/security.xml"

fos_user_resetting:
    resource: "@FOSUserBundle/Resources/config/routing/resetting.xml"
    prefix: resetting


bazookas_admin:
    resource: "@BazookasAdminBundle/Resources/config/routing.yml"
    prefix:   /{_locale}/admin
    defaults:
        _locale: "%locale%"
    requirements:
        _locale: "%requiredlocales%"
```

## FOSUserBundle

This bundle extends the FOS user bundle for authentication. Login and register templates are overridden by this bundle.

Default fos user bundle configuration

```YAML
fos_user:
    db_driver: orm
    firewall_name: admin
    user_class: Bazookas\AdminBundle\Entity\CMSUser
    group:
        group_class: Bazookas\AdminBundle\Entity\CMSUserGroup
    from_email:
        address: noreply@bazookas.be
        sender_name: Bazookas
    resetting:
        token_ttl: 86400
        email:
            template:   FOSUserBundle:Resetting:email.txt.twig
            # from_email:
            #     address:        ...
            #     sender_name:    ...
    service:
        mailer: fos_user.mailer.default
    profile:
        form:
            type:               Bazookas\AdminBundle\Form\ProfileFormType
            validation_groups:  [Profile, Default]
    registration:
        confirmation:
            # from_email: # Use this node only if you don't want the global email address for the confirmation email
            #    address:        ...
            #    sender_name:    ...
            enabled:    false # change to true for required email confirmation
            template:   FOSUserBundle:Registration:email.txt.twig
        form:
            type:               Bazookas\AdminBundle\Form\RegistrationType
            name:               fos_user_registration_form
            validation_groups:  [Registration, Default]
```
