bazookas.filters = (function () {
  var self = {};
  var container = '.js-filter';

  self.init = function (context) {
    bazookas.main
      .findInContext('.js-filter', context)
      .initElements(initFilterForm)
    ;
  };

  function initFilterForm() {
    var $container = $(this);
    var $field = $container.find('select.js-filter-field');

    var $operatorInput = $('select.js-filter-operator');
    //Only set the value if its not empty otherwise the select2 shows an empty option in safari and chrome
    if ($operatorInput.data('initially-selected')) {
      $operatorInput.val($operatorInput.data('initially-selected')).trigger('change.select2');
    }

    $container
      // add a delegated handler because the dom can change
      .on('change', '.js-filter-value', valueChangeHandler)
      .find('.js-filter-value')
      .change()
    ;

    // trigger change on field as last, to prevent "initially-selected" from breaking things (and such)
    $field
      .change(filterFieldChangeHandler)
      .change()
    ;
  }

  function filterFieldChangeHandler(event) {
    var $target = $(event.target),
      $container = $target.closest(container),
      $selected = $target.find(':selected'),
      type = $selected.data('type'),
      $valueInput = $container.find('.js-filter-value'),
      $parent = $valueInput.parent()
    ;

    // make sure no select2 keeps lingering around
    if ($valueInput.hasClass('select2-hidden-accessible')) {
      $valueInput.select2('destroy');
    }

    switch (type) {
      case 'bool':
      case 'boolean':
        convertToBooleanType($parent, $valueInput);
        break;
      case 'choice':
        convertToChoiceType($parent, $valueInput, $selected.data('choices'));
        break;
      case 'text':
      default:
        convertToTextType($parent, $valueInput, type);
        break;
    }

    $container.find('.js-filter-type').val(type);

    updateOperators($container, $selected);
  }

  function convertToBooleanType($parent, $valueInput) {
    $parent.addClass('checkbox mar-all').removeClass('form-group');

    // when the type if different, create a new input element and put it in the dom
    if ($valueInput.prop('type') !== 'checkbox') {
      var checked = $valueInput.val() == 1;

      $('<input type="checkbox" class="js-filter-value">')
        .prop('checked',checked)
        .val(1)
        .attr('id', $valueInput.attr('id'))
        .attr('name', $valueInput.attr('name'))
        .replaceAll($valueInput)
      ;
    }
  }

  function convertToChoiceType($parent, $valueInput, choices) {
    $parent.addClass('form-group').removeClass('checkbox mar-all');

    // convert the possible allowed values to html options
    var htmlOptions = '';
    for (var label in choices) {
      if (choices.hasOwnProperty(label)) {
        htmlOptions += '<option value="' + choices[label] + '">' + label + '</option>';
      }
    }

    // create a select list with the correct id, name and value
    var $select = $('<select class="js-filter-value form-control">')
      .html(htmlOptions)
      .val($valueInput.val())
      .attr('id', $valueInput.attr('id'))
      .attr('name', $valueInput.attr('name'))
      .replaceAll($valueInput)
    ;

    // make it pretty, but wait a little to make sure the dom is already updated
    setTimeout(function () {
      bazookas.form.initSelect2.call($select);
    }, 20);
  }

  function convertToTextType($parent, $valueInput, type) {
    $parent.addClass('form-group').removeClass('checkbox mar-all');

    // when the type if different, create a new input element and put it in the dom
    if ($valueInput.prop('type') !== type) {
      $('<input type="' + type + '" class="js-filter-value form-control">')
        .attr('id', $valueInput.attr('id'))
        .attr('name', $valueInput.attr('name'))
        .val($valueInput.val())
        .replaceAll($valueInput)
      ;
    }
  }

  function updateOperators($container, $selected) {
    //set the options for the operators input
    var $operatorInput = $container.find('select.js-filter-operator');
    var oldOperator = $operatorInput.val();
    var operators = $selected.data('operators');
    var operatorCount = 0;
    var options = '';

    for (var key in operators) {
      if (operators.hasOwnProperty(key)) {
        options += '<option value="' + operators[key] + '"';
        if (operators[key] == oldOperator) {
          options += ' selected="selected"'
        }
        options += '>' + key + '</option>';
        ++operatorCount;
      }
    }
    $operatorInput.html(options);
    if (!$operatorInput.val()) {
      $operatorInput.val($operatorInput.find('option').first().val());
    }

    // refresh the input select2
    if ($operatorInput.hasClass('select2-hidden-accessible')) {
      $operatorInput.select2('destroy');
      bazookas.form.initSelect2.call($operatorInput);
    }

    if (operatorCount > 1) {
      $operatorInput.parent('.form-group').css('display', '');
    }
    else {
      //Need to hide the parent because otherwise the select2 dropdown won't be hidden
      $operatorInput.parent('.form-group').hide();
    }
  }

  function valueChangeHandler(event) {
    var $target = $(event.target),
        $container = $target.closest(container),
        $selected = $container.find('.js-filter-field').find(':selected')
    ;

    if ($selected.data('type') == 'bool') {
      if ($(event.currentTarget).is(':checked')) {
        $(event.currentTarget).val(1);
      }
    }
  }

  return self;
})();
