<?php

namespace Bazookas\AdminBundle\Tests\FunctionalTests;

use Bazookas\AdminBundle\Controller\ListBaseController;
use Bazookas\AdminBundle\Entity\CMSUser;
use Bazookas\AdminBundle\Entity\CMSUserGroup;
use Bazookas\AdminBundle\Tests\TestCase\AdminTestCase;

/**
 * @deprecated to be removed in v2
 */
class AdminUserComponentFunctionalTest extends AdminTestCase
{

  /**
   * Test that pages load successfully when logged in as admin
   * @return array
   */
  public function getUrls() {
    $urls = [
      $this->getUrl('cms_user_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => ListBaseController::ACTION_LIST
      ]),
      $this->getUrl('cms_user_group_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => ListBaseController::ACTION_LIST
      ])
    ];

    $parentUrls = parent::getUrls();
    return array_merge($urls, $parentUrls);
  }

  public function testAddGroup() {
    list($client, $crawler) = $this->executeRequest('cms_user_group_admin', [
      '_locale' => $this->defaultLocale,
      '_action' => ListBaseController::ACTION_ADD
    ]);

    $form = $crawler->selectButton('submit')->form();
    $form['cms_user_group_admin[name]'] = 'test';
    $form['cms_user_group_admin[roles][0]']->tick();
    $client->submit($form);

    //Assert that the form submitted successfully
    $this->assertTrue($client->getResponse()->isRedirect());

    //Assert that the new item was added
    $entityManager = $client->getContainer()->get('doctrine.orm.entity_manager');
    $entityRepo = $entityManager->getRepository(CMSUserGroup::class);
    $entity = $entityRepo->findOneBy(['name' => 'test']);

    $this->assertTrue(!empty($entity));
  }

  public function testEditGroup() {
    list($client, $crawler) = $this->executeRequest('cms_user_group_admin', [
      '_locale' => $this->defaultLocale,
      '_action' => ListBaseController::ACTION_EDIT,
      '_id' => 1
    ]);

    $form = $crawler->selectButton('submit')->form();
    $form['cms_user_group_admin[name]'] = 'test2';
    $client->submit($form);

    //Assert that the form submitted successfully
    $this->assertTrue($client->getResponse()->isRedirect());

    //Assert that the item was edited
    $entityManager = $client->getContainer()->get('doctrine.orm.entity_manager');
    $entityRepo = $entityManager->getRepository(CMSUserGroup::class);
    $entity = $entityRepo->findOneBy(['name' => 'test']);

    $this->assertTrue(empty($entity));

    $entity = $entityRepo->findOneBy(['name' => 'test2']);
    $this->assertTrue(!empty($entity));
  }

  public function testAddUser() {
    list($client, $crawler) = $this->executeRequest('cms_user_admin', [
      '_locale' => $this->defaultLocale,
      '_action' => ListBaseController::ACTION_ADD
    ]);

    $form = $crawler->selectButton('submit')->form();
    $form['add_cms_user_admin[name]'] = 'test1';
    $form['add_cms_user_admin[surname]'] = 'test1';
    $form['add_cms_user_admin[email]'] = 'test@bazookas.be';
    $form['add_cms_user_admin[username]'] = 'test';
    $form['add_cms_user_admin[locale]']->select($this->defaultLocale);
    $form['add_cms_user_admin[plainPassword][first]'] = 'test';
    $form['add_cms_user_admin[plainPassword][second]'] = 'test';

    $client->submit($form);

    //Assert that the form submitted successfully
    $this->assertTrue($client->getResponse()->isRedirect());

    //Assert that the item was edited
    $entityManager = $client->getContainer()->get('doctrine.orm.entity_manager');
    $entityRepo = $entityManager->getRepository(CMSUser::class);
    $entity = $entityRepo->findOneBy(['name' => 'test1']);

    $this->assertTrue(!empty($entity));
  }

  public function testEditUser() {
    list($client, $crawler) = $this->executeRequest('cms_user_admin', [
      '_locale' => $this->defaultLocale,
      '_action' => ListBaseController::ACTION_EDIT,
      '_id' => 1
    ]);

    $form = $crawler->selectButton('submit')->form();
    $form['edit_cms_user_admin[name]'] = 'test2';
    $form['edit_cms_user_admin[surname]'] = 'test1';
    $form['edit_cms_user_admin[email]'] = 'test@bazookas.be';
    $form['edit_cms_user_admin[username]'] = 'test';
    $form['edit_cms_user_admin[locale]']->select($this->defaultLocale);

    $client->submit($form);

    //Assert that the form submitted successfully
    $this->assertTrue($client->getResponse()->isRedirect());

    //Assert that the item was edited
    $entityManager = $client->getContainer()->get('doctrine.orm.entity_manager');
    $entityRepo = $entityManager->getRepository(CMSUser::class);
    $entity = $entityRepo->findOneBy(['name' => 'test2']);

    $this->assertTrue(!empty($entity));
  }

  public function testRoleAdminUser() {
    $adminUser = $this->referenceRepository->getReference('user-admin1');

    $this->testPageIsSuccessful($adminUser, [
      $this->getUrl('cms_user_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => ListBaseController::ACTION_LIST
      ])
    ]);
  }

  public function testRequireRoleAdminUser() {
    $cmsUser = $this->referenceRepository->getReference('user-basic1');
    $this->testPageFails($cmsUser, [
      $this->getUrl('cms_user_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => ListBaseController::ACTION_LIST
      ]),
      $this->getUrl('cms_user_group_admin', [
        '_locale' => $this->defaultLocale,
        '_action' => ListBaseController::ACTION_LIST
      ])
    ]);
  }

}
