<?php
namespace Bazookas\AdminBundle\Util;

/**
 * Helper class to configure a single entity field
 **/
class ListField {
  /**
   * @var string the property name of the field to render
   **/
  public $field;

  /**
   * @var string the translation key to use in the table header
   **/
  public $label;

  /**
   * @var boolean|string whether or not the column is sortable, if string, used as field to sort
   **/
  public $sortable;

  /**
   * @var string path to the template to use for table cell
   **/
  public $template;

  /**
   * @var string optional css class to add to the td or th
   **/
  public $class;

  /**
   * @var boolean whether or not the column is orderable
   **/
  public $orderable;

  public function __construct() {
    $this->sortable = false;
    $this->orderable = false;
  }

  public function getField()
  {
    return $this->field;
  }

  public function setField($field)
  {
    $this->field = $field;

    return $this;
  }

  public function getLabel()
  {
    return $this->label;
  }

  public function setLabel($label)
  {
    $this->label = $label;

    return $this;
  }

  public function getSortable()
  {
    return $this->sortable;
  }

  public function setSortable($sortable)
  {
    $this->sortable = $sortable;

    return $this;
  }

  public function getTemplate()
  {
    return $this->template;
  }

  public function setTemplate($template)
  {
    $this->template = $template;

    return $this;
  }

  public function getClass()
  {
    return $this->class;
  }

  public function setClass($class)
  {
    $this->class = $class;

    return $this;
  }

  public function isOrderable()
  {
    return $this->orderable;
  }

  public function setOrderable($orderable)
  {
    $this->orderable = $orderable;

    return $this;
  }

  /**
   * Helper function for rendering, returns a string representation of the given value
   * @param string $value the original value
   * @return string parsed value
   **/
  public function getStringValue($value) {
    // REVIEW add more string conversions or use custom templates?
    if (is_array($value)) {
      return implode($value, ', ');
    }
    else if ($value instanceof \DateTime) {
      return $value->format('Y-m-d H:i:s');
    }
    else if ($value instanceof \Doctrine\ORM\PersistentCollection) {
      return '?';
    }
    else if (is_object($value)) {
      return '?';
    }
    return $value;
  }
}
