<?php

namespace Bazookas\CommonBundle\Entity\Base;

use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\User\EquatableInterface;
use Doctrine\ORM\Mapping as ORM;

/**
* @ORM\MappedSuperclass
*/
abstract class BaseUser extends BaseEntity implements UserInterface, EquatableInterface {

  /**
  * @ORM\Column(type="string")
  */
  private $username;

  /**
  * @ORM\Column(type="string")
  */
  private $password;

  /**
  * @ORM\Column(type="string")
  */
  private $salt;

  /**
  * @ORM\Column(type="array", nullable=true)
  */
  protected $roles;

  public function __construct($username, $password, $salt, array $roles)
  {
    parent::__construct();
    $this->username = $username;
    $this->password = $password;
    $this->salt = $salt;
    $this->roles = $roles;
  }

  public function getRoles()
  {
    return $this->roles;
  }

  public function setRoles($roles) {
    $this->roles = $roles;
    return $this;
  }

  public function getPassword()
  {
    return $this->password;
  }

  public function setPassword($password) {
    $this->password = $password;
    return $this;
  }

  public function getSalt()
  {
    return $this->salt;
  }

  public function setSalt($salt) {
    $this->salt = $salt;
    return $this;
  }

  public function getUsername()
  {
    return $this->username;
  }

  public function setUsername($name) {
    $this->username = $name;
    return $this;
  }

  public function eraseCredentials()
  {
    //TODO implement this
  }

  public function isEqualTo(UserInterface $user)
  {
    if (!$user instanceof BaseUser) {
      return false;
    }

    if ($this->password !== $user->getPassword()) {
      return false;
    }

    if ($this->salt !== $user->getSalt()) {
      return false;
    }

    if ($this->username !== $user->getUsername()) {
      return false;
    }

    return true;
  }
}
