<?php

namespace Bazookas\CommonBundle\Services\Util\HttpClient;

use Bazookas\CommonBundle\Exception\ConnectionException;

/**
 * @deprecated
 */
class HttpUtil {

  //Method constants
  const METHOD_HEAD = 'HEAD';
  const METHOD_GET = 'GET';
  const METHOD_POST = 'POST';
  const METHOD_PUT = 'PUT';
  const METHOD_PATCH = 'PATCH';
  const METHOD_DELETE = 'DELETE';
  const METHOD_PURGE = 'PURGE';
  const METHOD_OPTIONS = 'OPTIONS';
  const METHOD_TRACE = 'TRACE';
  const METHOD_CONNECT = 'CONNECT';

  //header constants
  const CONTENT_TYPE_JSON = 'application/json';
  const CONTENT_TYPE_XML = 'application/xml';
  const CONTENT_TYPE_WWW_FORM_URLENCODED = 'application/x-www-form-urlencoded';
  const CONTENT_TYPE_HTML = 'text/html';

  public static function getContentType(array $headers) {
    //get the content type header
    foreach($headers as $header) {
      if (strpos($header, 'Content-Type:') > -1) {
        $type = explode(':', $header);
        $type = $type[1];
        $type = explode(';', $type);
        $type = trim($type[0]);

        $mimeTypeMap = self::getMimeTypeMap();

        if ( isset($mimeTypeMap[$type]) ) {
          return $mimeTypeMap[$type];
        } else {
          throw new ConnectionException('Unknown content type in header: ' . $type);
        }
      }
    }

    throw new ConnectionException('Content type header could not be found');
  }

  protected static function getMimeTypeMap() {
    return [
      'application/json' => self::CONTENT_TYPE_JSON,
      'text/json' => self::CONTENT_TYPE_JSON,
      'application/xml' => self::CONTENT_TYPE_XML,
      'text/xml' => self::CONTENT_TYPE_XML,
      'application/x-www-form-urlencoded' => self::CONTENT_TYPE_WWW_FORM_URLENCODED,
      'text/html' => self::CONTENT_TYPE_HTML
    ];
  }
}
