## Choosing a service to select your data

When using the export-service, you have the option to provide the export-service with your own custom service to select data  from the database.

### Using your own service

The default service selects data from the database and applies the conditions you provided if necessary. Nothing more, nothing less. But there might be situations when you want to modify the data selected from the database before creating the file.

Say for example that you wanted to add an extra column to your export and make a sum of all the records that have the same value for a particular field. This would not be possible with the default service.

In order to use your own service, you need to make sure that your service implements the interface used by the default service.

Example:
```php
class MyCustomDataSelectionService implements DataSelectorInterface {
  
  public function getData(array $exportConfig) {
    
    ...
    
    return $exportData;
  }
}
```
When using your own service, you also need to make sure that your service returns the data to the export-service in the correct format. The data returned should contain the following:
```php
$exportData = [
  [
    'headers' => ['id', 'name', 'weight']
    'data' => [
      [1, "Just a string", 5],
      [2, "More data here", 3],
      [3, "Look, you get the point", 7],
      ...
    ],
    'worksheet' => 'My Sheet-name',
  ],
  [
    ...
  ],  
];
```
The headers and data are required, the worksheet field is optional and only used when exporting to excel. If you want a custom name for your sheet, you need to provide this, othewise the default values are used.

If you are exporting to .csv, the export-function will only use the first element in the array, as each element corresponds to a sheet, and CSV's can only have one sheet.
