<?php

namespace WebsiteBundle\Controller\Base;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use WebsiteBundle\Entity\Experiment;
use WebsiteBundle\Entity\Participant;
use WebsiteBundle\Entity\TestSession;

class BaseUserController extends Controller
{

  /**
   * @return Experiment
   */
  protected function getExperiment() {
    $experiment = $this->getUser()->getExperiment();
    if (!$experiment instanceof Experiment) {
      throw new AccessDeniedException('The user is not part of an experiment.');
    }

    return $experiment;
  }

  /**
   * @param Experiment $experiment
   * @return TestSession|null
   */
  protected function getNextSession(Experiment $experiment) {
    //Get the session
    $entityRepo = $this->get('doctrine.orm.entity_manager')->getRepository(TestSession::class);
    return $entityRepo->getNextSession($this->getUser(), $experiment);
  }

  protected function getBasicData() {
    $experiment = $this->getExperiment();
    $testSession = $this->getNextSession($experiment);

    return [
      'experiment' => $experiment,
      'testSession' => $testSession,
      'headerTitle' => 'PREVEND'
    ];
  }

  protected function handleAccessDeniedException(Request $request, $redirectTo = 'user_home') {
    //Log the user out
    $this->get('security.token_storage')->setToken(null);
    $request->getSession()->invalidate();

    //Add a flash message
    $translator = $this->get('translator');
    $message = $translator->trans('site.flash.user.notInExperiment');
    $this->addFlash('error', $message);

    return $this->redirectToRoute($redirectTo);
  }

}
