<?php

namespace WebsiteBundle\Controller\Site;

use Bazookas\APIFrameworkBundle\Controller\Base\BaseController;
use Bazookas\CommonBundle\Exception\NotImplementedException;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Constraints\All;
use Symfony\Component\Validator\Constraints\Collection;
use Symfony\Component\Validator\Constraints\NotBlank;

class AjaxController extends BaseController
{
  /**
   * @param Request $request
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   */
  public function submitPsychEducationQuestionsAction(Request $request) {
    $params = [
      'data' => [
        'constraint' =>  new All(new Collection([
            'fields' => [
              'panelId' => new NotBlank(),
              'responseIds' => new NotBlank()
            ]
          ])
        )
      ]
    ];

    $extraParams = new ParameterBag([
      'user' => $this->getUser()
    ]);

    return $this->handleRequest($request, $params, 'submitPsychEducationQuestions', Response::HTTP_OK, $extraParams);
  }

  public function submitTestResultsAction(Request $request) {
    $params = [
      'data' => [
        'constraint' =>  new All(new Collection([
            'fields' => [
              'trial' => new NotBlank(),
              'stimulus' => new NotBlank(),
              'answer' => [],
              'correctAnswer' => new NotBlank(),
              'reactionTime' => [],
              'interval' => new NotBlank(),
              'level' => new NotBlank(),
              'score' => new NotBlank(),
            ]
          ])
        )
      ],
      'inputType' => [
        'constraint' => new NotBlank()
      ]
    ];

    $extraParams = new ParameterBag([
      'user' => $this->getUser()
    ]);

    return $this->handleRequest($request, $params, 'submitTestResults', Response::HTTP_OK, $extraParams);
  }

  public function submitMoodQuestionsAction(Request $request) {
    $params = [
      'data' => [
        'constraint' =>  new All(new Collection([
            'fields' => [
              'questionId' => new NotBlank(),
              'answer' => new NotBlank()
            ]
          ])
        )
      ],
      'comment' => [
        'constraint' => new NotBlank(),
        'required' => false
      ]
    ];

    $extraParams = new ParameterBag([
      'user' => $this->getUser()
    ]);

    return $this->handleRequest($request, $params, 'submitMoodQuestions', Response::HTTP_OK, $extraParams);
  }

  public function markQuoteAsViewedAction(Request $request) {
    $params = [
      'unlockedQuoteId' => [
        'constraint' => new NotBlank()
      ]
    ];

    $extraParams = new ParameterBag([
      'user' => $this->getUser()
    ]);

    return $this->handleRequest($request, $params, 'markAsViewed', Response::HTTP_OK, $extraParams);
  }

}
