<?php

namespace WebsiteBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use WebsiteBundle\Security\Roles;

/**
 * @ORM\Entity(repositoryClass="WebsiteBundle\Repository\TestAnswerRepository")
 * @ORM\HasLifecycleCallbacks
 */
class TestAnswer extends BaseEntity {

  /**
   * @ORM\ManyToOne(targetEntity="Participant")
   * @var Participant
   */
  private $participant;

  /**
   * @ORM\ManyToOne(targetEntity="TestSession", inversedBy="answers")
   * @var TestSession
   */
  private $session;

  /**
   * @ORM\Column(type="integer")
   * @var integer the question number
   * @Assert\NotBlank
   */
  private $trial;

  /**
   * @ORM\Column(type="integer")
   * @var integer the digit they heard this time
   * @Assert\NotBlank
   */
  private $stimulus;

  /**
   * @ORM\Column(type="integer", nullable=true)
   * @var integer|null the answer the participant provided
   * @Assert\NotBlank
   */
  private $answer;

  /**
   * @ORM\Column(type="integer")
   * @var integer
   * @Assert\NotBlank
   */
  private $correctAnswer;

  /**
   * @ORM\Column(type="boolean")
   * @var boolean
   * @Assert\NotBlank
   */
  private $correct;

  /**
   * @ORM\Column(type="integer", nullable=true)
   * @var integer|null (in milliseconds)
   * @Assert\NotBlank
   */
  private $reactionTime;

  /**
   * @ORM\Column(type="integer", name="`interval`")
   * @var integer (in milliseconds)
   * @Assert\NotBlank
   */
  private $interval;

  /**
   * @ORM\Column(type="integer")
   * @var integer
   * @Assert\NotBlank
   */
  private $consecutiveCorrect;

  /**
   * @ORM\Column(type="integer")
   * @var integer
   * @Assert\NotBlank
   */
  private $consecutiveWrong;

  /**
   * @ORM\Column(type="integer")
   * @var integer
   * @Assert\NotBlank
   */
  private $level;

  /**
   * @ORM\Column(type="integer")
   * @var integer
   */
  private $score;

  public function getParticipant()
  {
    return $this->participant;
  }

  public function setParticipant(Participant $participant)
  {
    $this->participant = $participant;

    return $this;
  }

  public function getSession()
  {
    return $this->session;
  }

  public function setSession($session)
  {
    $this->session = $session;

    return $this;
  }

  public function getTrial()
  {
    return $this->trial;
  }

  public function setTrial($trial)
  {
    $this->trial = $trial;

    return $this;
  }

  public function getStimulus()
  {
    return $this->stimulus;
  }

  public function setStimulus($stimulus)
  {
    $this->stimulus = $stimulus;

    return $this;
  }

  public function getAnswer()
  {
    return $this->answer;
  }

  public function setAnswer($answer)
  {
    $this->answer = $answer;

    return $this;
  }

  public function getCorrectAnswer()
  {
    return $this->correctAnswer;
  }

  public function setCorrectAnswer($correctAnswer)
  {
    $this->correctAnswer = $correctAnswer;

    return $this;
  }

  public function isCorrect()
  {
    return $this->correct;
  }

  public function setCorrect($correct)
  {
    $this->correct = $correct;

    return $this;
  }

  public function getReactionTime()
  {
    return $this->reactionTime;
  }

  public function setReactionTime($reactionTime)
  {
    $this->reactionTime = $reactionTime;

    return $this;
  }

  public function getInterval()
  {
    return $this->interval;
  }

  public function setInterval($interval)
  {
    $this->interval = $interval;

    return $this;
  }

  public function getConsecutiveCorrect()
  {
    return $this->consecutiveCorrect;
  }

  public function setConsecutiveCorrect($consecutiveCorrect)
  {
    $this->consecutiveCorrect = $consecutiveCorrect;

    return $this;
  }

  public function getConsecutiveWrong()
  {
    return $this->consecutiveWrong;
  }

  public function setConsecutiveWrong($consecutiveWrong)
  {
    $this->consecutiveWrong = $consecutiveWrong;

    return $this;
  }

  public function getLevel()
  {
    return $this->level;
  }

  public function setLevel($level)
  {
    $this->level = $level;

    return $this;
  }

  /**
   * @return int
   */
  public function getScore()
  {
    return $this->score;
  }

  /**
   * @param int $score
   * @return TestAnswer
   */
  public function setScore($score)
  {
    $this->score = $score;

    return $this;
  }

  /**
   * @inheritdoc
   */
  public function getRoleRequiredForView() {
    return Roles::ROLE_CMS_USER;
  }
}
