<?php

namespace WebsiteBundle\Form\Admin;

use Bazookas\AdminBundle\Form\Type\NestedFormType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use WebsiteBundle\Entity\PsychEducationPanel;

class PsychEducationPanelAdminType extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $experiment = $options['experiment'];
    $experimentIsActive = !empty($experiment) ? $experiment->getSessionCount() > 0 : false;

    $builder
      ->add('type', ChoiceType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.type',
        'choices' => [
          'admin.entities.psychEducationPanel.enums.TYPE_TEXT' => PsychEducationPanel::TYPE_TEXT,
          'admin.entities.psychEducationPanel.enums.TYPE_RADIO_QUESTION' => PsychEducationPanel::TYPE_RADIO_QUESTION,
          'admin.entities.psychEducationPanel.enums.TYPE_CHECKBOX_QUESTION' => PsychEducationPanel::TYPE_CHECKBOX_QUESTION,
        ],
        'attr' => [
          'class' => 'js-psych-education-panel-type select2 form-control'
        ]
      ])
      ->add('title', TextType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.title'
      ])
      ->add('text', TextareaType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.text',
        'required' => false
      ])
      ->add('question', TextareaType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.question',
        'required' => false,
        'conditionallyVisible' => [
          '.js-psych-education-panel-type' => '$element.val() != "'.PsychEducationPanel::TYPE_TEXT.'"'
        ],
        'conditionalScope' => '.js-nested-form-item'
      ])
      ->add('responses', NestedFormType::class, [
        'label' => 'admin.entities.psychEducationPanel.fields.responses',
        'entry_type' => PsychEducationResponseAdminType::class,
        'conditionallyVisible' => [
          '.js-psych-education-panel-type' => '$element.val() != "'.PsychEducationPanel::TYPE_TEXT.'"'
        ],
        'conditionalScope' => '.js-nested-form-item',
        'allow_delete' => !$experimentIsActive
      ])
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => PsychEducationPanel::class,
      'experiment' => null,
      'translation_domain' => 'admin'
    ]);
  }
}
