<?php

namespace Bazookas\AdminBundle\AdminElements\Elements;

use Bazookas\AdminBundle\AdminElements\Base\BaseAdminElement;

class GraphElement extends BaseAdminElement {

  //graph types
  const TYPE_RADAR = 'radar';
  const TYPE_BAR = 'bar';
  const TYPE_HORIZONTAL_BAR = 'horizontalBar';
  const TYPE_PIE = 'pie';
  const TYPE_DOUGHNUT = 'doughnut';

  public $data;

  public $graphOptions;

  public $type;

  public function __construct($children = [], $options = []) {
    parent::__construct($children, $options);
    $this->type = $this->options['type'];
    $this->graphOptions = $this->options['options'];
    $this->data = $this->options['data'];
  }

  protected function getDefaultOptions() {
    $defaults = parent::getDefaultOptions();
    return array_merge($defaults, [
      'template' => 'BazookasAdminBundle:AdminElements\Elements:graphElement.html.twig',
      'class' => 'canvas-responsive js-graph',
      'type' => self::TYPE_BAR,
      'options' => [],
      'data' => [],
      'icon' => 'fa-pie',
      'iconClass' => 'fa'
    ]);
  }

  public function getData()
  {
    return $this->data;
  }

  public function setData($data)
  {
    $this->data = $data;

    return $this;
  }

  public function getGraphOptions()
  {
    return $this->graphOptions;
  }

  public function setGraphOptions($graphOptions)
  {
    $this->graphOptions = $graphOptions;

    return $this;
  }

  public function getType()
  {
    return $this->type;
  }

  public function setType($type)
  {
    $this->type = $type;

    return $this;
  }

  public static function generateDefaultDataDefinition($type) {
    switch($type) {
      case self::TYPE_RADAR:
        return array(
          'labels' => ['a Label'],
          'datasets' => [
            [
              'label' => 'My First dataset',
              'backgroundColor' => 'rgba(179,181,198,0.2)',
              'borderColor' => "rgba(179,181,198,1)",
              'pointBackgroundColor' => "rgba(179,181,198,1)",
              'pointBorderColor' => "#fff",
              'pointHoverBackgroundColor' => "#fff",
              'pointHoverBorderColor' => "rgba(179,181,198,1)",
              'data' => [13, 15, 45, 72, 49],
            ]
          ]
        );
      case self::TYPE_BAR:
        return array(
          'labels' => ['a Label'],
          'datasets' => [
            [
              'label' => 'My First dataset',
              'backgroundColor' => [
                'rgba(255, 99, 132, 0.2)',
                'rgba(54, 162, 235, 0.2)',
                'rgba(255, 206, 86, 0.2)',
                'rgba(75, 192, 192, 0.2)',
                'rgba(153, 102, 255, 0.2)',
                'rgba(255, 159, 64, 0.2)'
              ],
              'borderColor' => [
                'rgba(255,99,132,1)',
                'rgba(54, 162, 235, 1)',
                'rgba(255, 206, 86, 1)',
                'rgba(75, 192, 192, 1)',
                'rgba(153, 102, 255, 1)',
                'rgba(255, 159, 64, 1)'
              ],
              'borderWidth' => 1,
              'data' => [65, 59, 80, 81, 56, 55, 40]
            ]
          ]
        );
      case self::TYPE_PIE:
      case self::TYPE_DOUGHNUT:
        return array(
          'labels' => ['a Label', 'another', 'and another'],
          'datasets' => [
            [
              'label' => 'My First dataset',
              'backgroundColor' => [
                'rgba(255, 99, 132, 0.2)',
                'rgba(54, 162, 235, 0.2)',
                'rgba(255, 206, 86, 0.2)',
                'rgba(75, 192, 192, 0.2)',
                'rgba(153, 102, 255, 0.2)',
                'rgba(255, 159, 64, 0.2)'
              ],
              'borderColor' => [
                'rgba(255,99,132,1)',
                'rgba(54, 162, 235, 1)',
                'rgba(255, 206, 86, 1)',
                'rgba(75, 192, 192, 1)',
                'rgba(153, 102, 255, 1)',
                'rgba(255, 159, 64, 1)'
              ],
              'data' => [65, 59, 80]
            ]
          ]
        );
      default:
        throw new \Exception('Unknown graph type. A default data definition could not be generated. Please add it senpai!!! (◕ᴗ◕✿)');
    }
  }
}
