<?php

namespace Bazookas\AdminBundle\DependencyInjection;

use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;
use Symfony\Component\Config\Definition\Builder\NodeDefinition;

/**
 * This is the class that validates and merges configuration from your app/config files.
 *
 * To learn more see {@link http://symfony.com/doc/current/cookbook/bundles/configuration.html}
 */
class Configuration implements ConfigurationInterface
{
    /**
     * {@inheritdoc}
     */
    public function getConfigTreeBuilder()
    {
        $treeBuilder = new TreeBuilder();
        $rootNode = $treeBuilder->root('bazookas_admin');

        // building recursive config, as seen here
        // https://gist.github.com/Lumbendil/3249173
        $this->buildMenuNode(
          $rootNode
            ->children()
              ->arrayNode('menu')
                ->children()
                  ->arrayNode('main')
                    ->defaultValue(array())
                    ->prototype('array')
        )
        // NOTE very confusing: pass node to function before calling "end" methods
                ->end()
              ->end()
            ->end()
          ->end()
          ->arrayNode('audit')
            ->children()
              ->arrayNode('logging')
                ->children()
                  ->scalarNode('logClass')
                    ->isRequired()
                    ->cannotBeEmpty()
                  ->end()
                ->end()
              ->end()
              ->booleanNode('enableVersioning')
                ->defaultFalse()
              ->end()
            ->end()
          ->end()
          ->arrayNode('notifications')
            ->children()
              ->arrayNode('notificationEntities')
                ->children()
                  ->scalarNode('CMSNotification')->end()
                  ->scalarNode('EmailNotification')->end()
                ->end()
              ->end()
            ->end()
          ->end()
        ->end()
        ;

        return $treeBuilder;
    }

    // use this function to walk over child nodes and check if they are valid
    private function evaluateChildren(&$child, $name)
    {
      $child = $this->getMenuNode($name)->finalize($child);
    }

    // build the subtrees
    protected function getMenuNode($name = '')
    {
      $treeBuilder = new TreeBuilder();
      $definition = $treeBuilder->root($name);
      $this->buildMenuNode($definition);
      return $definition->getNode(true);
    }

    // build a single menu item
    private function buildMenuNode(NodeDefinition $node) {
      // TODO add validation??
      return $node
        ->addDefaultsIfNotSet()
        ->children()
          ->scalarNode('type')->isRequired()->end()
          ->scalarNode('label')->isRequired()->end()
          ->scalarNode('route')->end()
          ->scalarNode('icon')->end()
          ->scalarNode('description')->end()
          ->arrayNode('roles')
            ->prototype('scalar')
            ->end()
          ->end()
          ->variableNode('children')
              ->defaultValue(array())
              ->validate()->ifTrue(function($element) { return !is_array($element); })->thenInvalid('The children element must be an array.')->end()
              ->validate()->always(function($children) {array_walk($children, array($this, 'evaluateChildren'));return $children;})->end()
          ->end()
        ->end()
      ;
    }
}
