<?php

namespace Bazookas\AdminBundle\Form;

use Bazookas\AdminBundle\Entity\CMSUserGroup;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextType;

class CMSUserGroupAdminType extends AbstractType
{
  private $roleHierarchy;

  public function __construct($roleHierarchy) {
    $this->roleHierarchy = $roleHierarchy;
  }

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('name', TextType::class, array(
        'label' => 'admin.entities.CMSUserGroup.fields.name'
      ))
      ->add('roles', ChoiceType::class, array(
        'label' => 'admin.entities.CMSUserGroup.fields.roles',
        'choices' => $this->getRoleChoices(),
        'multiple' => true,
        'expanded' => true
      ))
    ;
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'data_class' => CMSUserGroup::class,
      'translation_domain' => 'admin'
    ));
  }

  //TODO somehow show the role hierarchy in the choices as well
  public function getRoleChoices()
  {
    $roles = [];
    $roles = $this->getRoles($this->roleHierarchy, $roles);
    $roles = array_unique($roles);

    $choices = [];
    foreach($roles as $role) {
      $choices[$role] = $role;
    }

    return $choices;
  }

  private function getRoles($rolesHierarcy, $roles) {
    foreach($rolesHierarcy as $key => $role) {
      if (is_array($role)) {
        $roles[] = $key;
        $roles = $this->getRoles($role, $roles);
      } else {
        $roles[] = $role;
      }
    }

    return $roles;
  }

}
