## Enabling registration for CMSusers

Follow the steps below to enable registration for the CMS.

### Routing.yml

add the following to `app/config/routing.yml`:

```yaml
fos_user_register:
    resource: "@FOSUserBundle/Resources/config/routing/registration.xml"
    prefix: /register
```

and enable the interface by setting the following twig global:

```yaml
twig:
    globals:
        registrationEnabled: true
```

### The registration form

The AdminBundle provides a default registration form. You can also provide your own form. For example lets assume we have the following form class:
 
```php
<?php

namespace WebsiteBundle\Form;

use ApiBundle\Entity\YumCMSUser;
use ApiBundle\Security\Roles;
use FOS\UserBundle\Form\Type\RegistrationFormType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\OptionsResolver\OptionsResolver;

class TeacherRegistrationForm extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {

    $builder
      ->add('name', TextType::class, [
        'label' => 'admin.registration.new.form.name'
      ])
      ->add('surname', TextType::class, [
        'label' => 'admin.registration.new.form.surname'
      ])
      ->add('school', TextType::class, [
        'label' => 'admin.registration.new.form.school'
      ])
      ->addEventListener(FormEvents::POST_SUBMIT, [$this, 'onPostSubmit']);
    ;
  }
  
  /**
   * @param FormEvent $event
   */
  public function onPostSubmit(FormEvent $event) {
    $data = $event->getForm()->getData();
  
    /** @var YumCMSUser $data */
    $data->setRoles([
      Roles::ROLE_TEACHER
    ]);
  }

  public function getParent()
  {
    return RegistrationFormType::class;
  }

  public function getBlockPrefix()
  {
    return 'teacher_registration';
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults([
      'data_class' => YumCMSUser::class,
      'translation_domain' => 'admin',
    ]);
  }
}
``` 
 
You must declare your form by overriding the `services.yml` definition for the default form:

```yaml
bazookas.admin.form.type.RegistrationType:
    class: Bazookas\AdminBundle\Form\RegistrationType
    tags:
        - { name: form.type }
```

and in `config.yml` declare which **form*** will be used and which **entity** will be used:

```yaml
fos_user:
    user_class: ApiBundle\Entity\CMSUser
    registration:
        form:
            type: WebsiteBundle\Form\TeacherRegistrationForm
```

### More information on this

- [official docs on overriding forms](https://symfony.com/doc/master/bundles/FOSUserBundle/overriding_forms.html)
