## Dashboard

The dashboard component allows you to create a page which can contain a variety of widgets. It's main use is for the creation of dashboards or statistics pages.

### Start by creating a controller

Create a controller by extending the `getProfileElement`. The base controller contains the basic functionality necessary to render a dashboard. A dashboard works by adding dashboard elements to a dashboard configuration. Below is a sample implementation of a dashboard controller:

```php
<?php

namespace WebsiteBundle\Controller;

use APIBundle\Entity\AppUser;
use Bazookas\AdminBundle\Controller\Base\DashboardBaseController;
use Bazookas\AdminBundle\Util\Dashboard\Elements\AlertElement;
use Bazookas\AdminBundle\Util\Dashboard\Elements\RowContainer;
use Symfony\Component\HttpFoundation\Request;

class GeneralStatsDashboardController extends DashboardBaseController
{

  function __construct() {
    // make sure parent construct is called!
    parent::__construct();

    $this->config->setTitle('admin.views.generalStats.name');
  }

  protected function hasAccess() {
    return $this->isGranted('ROLE_ADMIN');
  }

  protected function buildPage(Request $request) {
    $dataService = $this->get('websiteBundle.dataService.StatisticsDataService');
    $statsContainer = $dataService->getProfileElement($request->getLocale());

    $this->config
      ->addElement($statsContainer)
    ;
  }
}
```

In this example we add `getProfileElement` to the dashboard, below is what the getProfileElement function might could look like:

```php
public function getProfileElement(AppUser $user)
  {
    return new ProfileElement(
      [
        new ListContainer(
          [
            new TextElement([], [
              'value' => $user->getStoryMode(),
              'label' => 'admin.entities.appUser.fields.storyMode',
              'colClass' => 'col-xs-4 pad-all',
              'template' => 'WebsiteBundle:Dashboard\\AppUser:storyModeElement.html.twig',
            ]),
            new TextElement([], [
              'value' => $user->getCurrentTier() + 1,
              'label' => 'admin.entities.appUser.fields.currentTier',
              'colClass' => 'col-xs-4 pad-all',
            ]),
            new TextElement([], [
              'value' => $user->getRankScore(),
              'label' => 'admin.entities.appUser.fields.rankScore',
              'colClass' => 'col-xs-4 pad-all',
            ]),
          ]
        ),
        new ListContainer(
          [
            new TextElement([], [
              'value' => $user->getRank(),
              'label' => 'admin.entities.appUser.fields.rank',
              'colClass' => 'col-xs-4 pad-all',
            ]),
            new TextElement([], [
              'value' => $user->getGamesPlayed(),
              'label' => 'admin.entities.appUser.fields.gamesPlayed',
              'colClass' => 'col-xs-4 pad-all',
            ]),
          ]
        ),
      ],
      [
        'image' => new ImageElement([], [
          'avatarId' => $user->getAvatarId(),
          'class' => 'img-md img-circle mar-btm',
          'template' => 'WebsiteBundle:Dashboard\\AppUser:imageElement.html.twig',
        ]),
        'name' => $user->getUsername(),
        'role' => null,
      ]
    );
  }
```
As you can see we are directly adding a profile element. This element in turn contains some container whom in turn contain more elements. A container is something that holds a group of elements (like a parent `div` might contain one or more child `div`s). And everything can be nested ain't that dandy!
