## ToolTipExtension

This component allows you to add tooltip functionality to your form fields. 

### Getting started

```php
<?php

namespace WebsiteBundle\Form\Admin;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;

class MyForm extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('name', TextType::class, [
        'label' => 'admin.entities.experiment.fields.name',
        'tooltip_title' => 'The title',
        'tooltip_content' => 'The content of the tooltip',
        'tooltip_type' => 'tooltip',
        'tooltip_placement' => 'right',
        'tooltip_trigger' => 'hover'
      ])
    ;
  }
}
```

The tooltip will be added if a `tooltip_title` is defined. The form item can be given the following tooltip configurations:

- `tooltip_title` => The title
- `tooltip_content` => A longer content text, will only be used with tooltip of the type `popover`
- `tooltip_type` => the type of tooltip, accepted values are `popover` or `tooltip`; defaults to `tooltip`
    - type **tooltip**:
    ![tooltip_example](./Images/tooltip_example.png)
    - type **popover**:
    ![popover_example](./Images/popover_example.png)
- `tooltip_placement` => where should the tooltip appear? accepted values are `top`, `right`, `bottom`, `left`; defaults to `top`
- `tooltip_trigger` => what triggers the tooltip? defaults to `hover`
