bazookas.listTable = (function () {
  var self = {},
    initialized = false;

  self.init = function (context) {
    bazookas.main
      .findInContext('.js-ajax-pageable', context)
      .initElements(initAjaxPageable);

    bazookas.main
      .findInContext('.list-table-editable', context)
      .initElements(initInlineEditable);

    bazookas.main
      .findInContext('.js-collapse-item-button', context)
      .initElements(initListTableCollapsible);
  };

  function initAjaxPageable() {
    $(this).click(ajaxClickHandler);
  }

  function initInlineEditable() {
    $(this).click(inlineEditClickHandler)
  }

  function ajaxClickHandler(event) {
    event.preventDefault();

    var $this = $(this);

    $.ajax({
      url: $this.data('url'),
      success: function (response) {
        var $response = $(response);

        $response.hide();
        $this.closest('.js-list-table-panel').replaceWith($response.fadeIn());
        bazookas.main.init($response);
        $(window).scrollTop($response.offset().top - 15);

        //set the page history
        window.history.pushState({}, '', $this.prop('href'));
      },
      error: function (jqXHR, textStatus, errorThrown) {
        //fallback to a normal page refresh
        window.location = $this.prop('href');
      }
    })
  }

  function inlineEditClickHandler(event) {
    var $target = $(event.target);
    $target.addClass('list-table-no-hover');

    //Disable the event listener
    $target.off();

    //Store the unedited data
    var oldData = $target.text().trim();
    $target.data('old-data', oldData);

    //Add the inline form
    var input;
    if ($target.data('field-type') === 'textarea') {
      input = '<textarea class="form-control js-inline-input">' + oldData + '</textarea>';
    } else {
      input = '<input type="' + $target.data('field-type') + '" class="form-control js-inline-input" value="' + oldData + '" />';
    }

    $target.empty().html($('<div class="form-group">' + input + '<div class="pull-right"><button class="btn btn-danger btn-inline-cancel js-inline-cancel"><i class="fa fa-times" aria-hidden="true"></i></button><button class="btn btn-success btn-inline-submit js-inline-submit"><i class="fa fa-check" aria-hidden="true"></i></button></div></div>')).focus();

    $target.find('.js-inline-submit').click(inlineSubmitHandler);
    $target.find('.js-inline-cancel').click(inlineCancelHandler);
  }

  function inlineSubmitHandler(event) {
    var $parent = $(event.target).closest('.list-table-editable'),
        id = $parent.data('id'),
        field = $parent.data('field'),
        $input = $parent.find('.js-inline-input'),
        value = $input.val();

    $input.prop('disabled', true);

    $.ajax({
      url: $parent.data('url'),
      type: "POST",
      data: JSON.stringify({
        id: id,
        field: field,
        value: value
      }),
      contentType: "application/json; charset=utf-8",
      dataType: "json",
      success: function (response) {
        removeEditForm($parent, value);

        //Highlight the row for a set time to indicate that the item has been updated
        var $row = $parent.closest('tr').addClass('bg-success');

        setTimeout(function() {
          $row.removeClass('bg-success');
        }, 3500);
      },
      error: function (msg) {
        $input.prop('disabled', false);
        $.niftyNoty({
          type: "danger",
          container : "floating",
          title : trans('listTable.inlineEdit.error.title'),
          message : trans('listTable.inlineEdit.error.text'),
          closeBtn : true,
          timer : 4000
        });

        console.error(msg);
      }
    });
  }

  function inlineCancelHandler(event) {
    var $parent = $(event.target).closest('.list-table-editable');
    removeEditForm($parent, $parent.data('old-data'));
  }

  function removeEditForm($parent, text) {
    $parent.empty().text(text);
    $parent.click(inlineEditClickHandler);
    $parent.removeClass('list-table-no-hover');
  }

  function initListTableCollapsible() {
    $(this).click(listTableCollapsibleHandler)
  }

  function listTableCollapsibleHandler(event) {
    var $target = $(event.target);
    var $sibling = $target.siblings('.js-collapse-item');

    var $element = $('tr#'+$sibling.data('id'));
    if ($element.length > 0) {
      $element.remove();
    } else {
      var $row = $target.closest('tr');
      var columnCount = $row.children('td').length;

      $('<tr id="'+$sibling.data('id')+'"><td colspan="'+columnCount+'">'+$sibling.html()+'</td></tr>').insertAfter($row);
    }
  }

  return self;
})();
