<?php
namespace Bazookas\AdminBundle\Twig;

use Symfony\Component\HttpFoundation\Request;

/**
 * @deprecated to be removed in v2
 *
 * Class MenuExtension
 * @package Bazookas\AdminBundle\Twig
 */
class MenuExtension extends \Twig_Extension {

  private $menuConfig;
  private $mainMenu;
  private $securityAuthorizationChecker;
  private $accessMap;
  private $router;
  // private $requestStack;
  private $request;

  public function __construct($router, $securityAuthorizationChecker, $accessMap, $requestStack, $menuConfig){
    $this->router = $router;
    $this->securityAuthorizationChecker = $securityAuthorizationChecker;
    $this->accessMap = $accessMap;
    $this->menuConfig = $menuConfig;
    $this->request = $requestStack->getCurrentRequest();
  }

  /**
   * Get Twig functions defined in this extension.
   *
   * @return array
   */
  public function getFunctions()
  {
    return array(
      new \Twig_SimpleFunction('get_admin_main_menu', array($this, 'getAdminMainMenu')),
    );
  }

  /**
   * Return the admin menu MenuBuilder.
   *
   * @return MenuBuilder
   */
  public function getAdminMainMenu()
  {
    if (isset($this->mainMenu)) {
      return $this->mainMenu;
    }

    // prepare the menu for rendering in twig
    $menu = array();
    if (!empty($this->menuConfig['main'])) {
      // a little trickery, use the submenu function to parse the root menu
      $menu = $this->parseSubmenu(array(
        'children' => $this->menuConfig['main'])
      )['children'];
    }

    // store the menu for later use
    $this->mainMenu = $menu;
    return $menu;
  }

  private function parseMenuItem($item) {
    $doAccessCheck = TRUE;
    if (!empty($item['roles'])) {
      if (!$this->securityAuthorizationChecker->isGranted($item['roles'])) {
        return FALSE;
      }
      $doAccessCheck = FALSE;
    }

    // it is a link with a route, build the path and do access checks
    switch ($item['type']) {
      case 'link':
        return $this->parseLinkItem($item, $doAccessCheck);
      case 'header':
        return $item;
      case 'submenu':
        return $this->parseSubmenu($item);
    }
  }
  private function parseLinkItem($item, $doAccessCheck) {
    $item['path'] = $this->router->generate($item['route']);

    // check if the item's route is equal to the current request
    // and flag it as an active menu item
    if (!empty($this->request) && $this->request->get('_route') === $item['route']) {
      $item['active'] = true;
    }

    // check if user has access to this path
    if ($doAccessCheck) {
      if (!$this->checkPathAccess($item['path'])) {
        return FALSE;
      }
    }
    return $item;
  }
  private function parseSubmenu($item) {
    // create a new array to make it possible to skip children to which no access is granted
    $parsedChildren = array();
    foreach ($item['children'] as $key => $child) {
      $parsedChild = $this->parseMenuItem($child);
      if ($parsedChild) {
        $parsedChildren[] = $parsedChild;
        if(isset($parsedChild['active']) && $parsedChild['active']) {
          $item['active'] = TRUE;
        }
      }
    }

    switch (count($parsedChildren)) {
      // when no children, hide the parent element
      case 0:
        return null;
      // when 1 child, move it up one level
      case 1:
        return $parsedChildren[0];
    }

    $item['children'] = $parsedChildren;
    return $item;
  }

  public function getName() {
    return 'bazookas_admin_menu_extension';
  }


  /**
   * This function gets the roles that have access to a certain path, using the security.access_map service
   * @param string $path The path to check access for
   * @return boolean TRUE if the user has access, FALSE if not
   **/
  public function checkPathAccess($path){
    // build a request based on path to check access
    $request = Request::create($path,'GET');

    // get the roles that have access to this path
    list($roles,$channel) = $this->accessMap->getPatterns($request);

    // check if access is granted
    return $this->securityAuthorizationChecker->isGranted($roles);
  }
}
