# AppSettings

There is a way to let the API framework handle app settings for you.

## CMS

Add the following route (optionally prefixed):

```
bazookas_api_framework_app_settings_admin:
    path:     /app-settings/{_action}/{_id}
    defaults:
      _controller: BazookasAPIFrameworkBundle:Admin\AppSettingsAdmin:index
      _action: ~
      _id: ~
```

Add the menu item:

```php
// or $this->elements[] = new MenuActionElement(...);
$container->addChild(new MenuActionElement([], [
  'label' => 'admin.entities.appSettings.namePlural',
  'route' => 'bazookas_api_framework_app_settings_admin',
  'iconClass' => 'fa fa-gears',
  'roles' => [Roles::ROLE_SUPER_ADMIN],
]));
```

The rest will be handled by the api framework.

## Configuring your project

1. Create an entity which extends `Bazookas\APIFrameworkBundle\Entity\Base\BaseAppSettings`

```
<?php
namespace APIBundle\Entity;

use Bazookas\APIFrameworkBundle\Entity\Base\BaseAppSettings;
use Doctrine\ORM\Mapping as ORM;

/**
 * @ORM\Entity(
 *   repositoryClass="Bazookas\APIFrameworkBundle\Repository\AppSettingsAdminRepository"
 * )
 * @ORM\HasLifecycleCallbacks
 */
class AppSettings extends BaseAppSettings
{
  // no real need to add anything specific, unless you need extra settingsability
}

```

2. Add the following route:

```
api_v1_get_settings:
  path: /settings
  methods: [GET]
  defaults:
    _controller: BazookasAPIFrameworkBundle:AppSettings:get
```

3. Let the framework know which entity to use (in config.yml):

```
bazookas_api_framework:
  app_settings:
    entity: APIBundle\Entity\AppSettings
```
