<?php

namespace Bazookas\APIFrameworkBundle\Services\Data;

use Bazookas\APIFrameworkBundle\Formatter\AppCopyFormatter;
use Bazookas\APIFrameworkBundle\Services\Data\Base\BaseDataService;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\QueryBuilder;
use Symfony\Component\HttpFoundation\ParameterBag;

// NOTE: this service is defined in BazookasAPIFrameworkExtension.php
class AppCopyDataService extends BaseDataService
{
  public function __construct(EntityManagerInterface $entityManager, $entityClass) {
    $this->repository = $entityManager->getRepository($entityClass);

    // don't call the parent constructor, because we can't be sure the repository extends the correct repository interface
    //parent::__construct($repository, null);
  }
  
  public function getAll(ParameterBag $params, ParameterBag $extraParams) {
    $newer = $params->get('newer');
    $locale = $extraParams->get('locale');

    /**
     * @var QueryBuilder $qb
     */
    $qb = $this->repository->createQueryBuilder('c');
    $qb
      ->addSelect('d')
      ->leftJoin('c.details', 'd', 'WITH', 'd.locale = :locale')
      ->setParameter('locale', $locale)
    ;

    if ($params->has('newer')) {
      $qb
        ->andWhere($qb->expr()->orX(
          $qb->expr()->gt('c.created', ':newer'),
          $qb->expr()->gt('c.modified', ':newer')
        ))
        ->setParameter('newer', $newer);
    }
    $result = $qb->getQuery()->getArrayResult();

    $formatter = new AppCopyFormatter();
    $result = $formatter->format($result);

    $lastModified = $this->getLastModified($newer, $result, true);
    if (!empty($lastModified)) {
      $lastModified = $lastModified->format('U');
    }

    // return the resulting settings
    return [
      'data' => $result,
      'lastModified' => $lastModified,
    ];
  }
}
