<?php

namespace Bazookas\APIFrameworkBundle\Services\Data\Base;

use Bazookas\APIFrameworkBundle\Exception\InvalidArgumentException;
use Bazookas\APIFrameworkBundle\Exception\ServiceException;
use Symfony\Bundle\FrameworkBundle\Templating\EngineInterface;
use Symfony\Component\HttpFoundation\ParameterBag;
use Symfony\Component\Translation\TranslatorInterface;

abstract class BaseAuthDataService extends BaseDataService
{
  protected $mailer;
  protected $templateEngine;
  protected $translator;
  protected $fromEmail;

  public function __construct($mailer, EngineInterface $templateEngine, TranslatorInterface $translator, $fromEmail) {
    $this->mailer = $mailer;

    $this->templateEngine = $templateEngine;
    $this->translator = $translator;
    $this->fromEmail = $fromEmail;
  }

  public function forgotPassword(ParameterBag $params, ParameterBag $extraParams) {
    if ($params->has('Email')) {
      $email = $params->get('Email');
    } else {
      $email = $params->get('email');
    }

    $user = $this->repository->findOneBy(array('email' => $email));

    if (empty($user)) {
      if ($extraParams->has('invalidUserExceptionCode')) {
        throw new InvalidArgumentException('The email address provided not valid.', $extraParams->get('invalidUserExceptionCode'));
      } else {
        throw new InvalidArgumentException('The email address provided not valid.');
      }
    }

    $user = $this->repository->forgotPassword($user);
    $this->sendForgotPasswordEmail($user);
  }

  /**
   * @param $user the user!
   * @return string a url where the user can reset their password
   */
  public abstract function getForgotPasswordLink($user);

  /**
   * @param $user the user!
   * @return string a url where the user can reset their password
   */
  public function sendForgotPasswordEmail($user) {
    if ($this->mailer instanceof \Swift_Mailer) {
      $link = $this->getForgotPasswordLink($user);

      $plainText = $this->getForgotPasswordPlainTextMessage($link, $user);
      $subject = $this->translator->trans('api.email.user.forgotPassword.subject', [], 'admin');
      $message = \Swift_Message::newInstance()
        ->setSubject($subject)
        ->setFrom(array($this->fromEmail => $this->translator->trans('api.email.user.forgotPassword.fromEmailName', [], 'admin')))
        ->setTo($user->getEmail())
        ->setBody($plainText, 'text/plain')
        ->addPart(
          $this->templateEngine->render(
            $this->getForgotPasswordTemplate(),
            array(
              'user' => $user,
              'link' => $link
            )
          ),
          'text/html'
        )
      ;

      $this->mailer->send($message);
    } else {
      throw new ServiceException('Unknown mailer service in ' . self::class);
    }
  }

  public function getForgotPasswordPlainTextMessage($link, $user) {
    $firstName = method_exists($user, 'getFirstName') ? $user->getFirstName() : '';
    $from = $this->translator->trans('api.email.user.forgotPassword.from', [], 'admin');

    return  $this->translator->trans('api.email.user.forgotPassword.body', [
       '%link%' => $link,
       '%firstName%' => $firstName,
       '%from%' => $from
    ], 'admin');
  }

  /**
   * @return string the location of the twig template to be used to render the HTML email
   */
  public abstract function getForgotPasswordTemplate();
}
