<?php

namespace Bazookas\CommonBundle\Controller\Base;

use Bazookas\CommonBundle\Form\BaseAppUserForgotPasswordForm;
use Symfony\Component\Form\FormError;
use Symfony\Component\HttpFoundation\Request;

abstract class BaseAuthController extends BaseController
{

  public abstract function getUserEntity();

  public function resetPasswordAction(Request $request)
  {
    $templateData = [];

    $resetHash = $request->query->get('hash');
    //check if we've been given a hash
    if (empty($resetHash)) {
      $this->addFlashMessage('error', 'common.entities.BaseAppUser.flash.hashInvalid');
    } else {
      $repo = $this
        ->getDoctrine()
        ->getManager()
        ->getRepository($this->getUserEntity());

      //TODO add instance of interface check here to prevent fatal errors, this on both repo and on the user

      $user = $repo->getUserByForgotPasswordHash($resetHash);

      if (empty($user)) {
        $this->addFlashMessage('error', 'common.entities.BaseAppUser.flash.hashInvalid');
      } else {
        if ($user->getForgotPasswordExpiry() < new \DateTime("now", new \DateTimeZone('UTC'))) {
          $this->addFlashMessage('error', 'common.entities.BaseAppUser.flash.hashExpired');
        } else {
          //check if a user was found
          $form = $this->createForm(BaseAppUserForgotPasswordForm::class, $user);
          $form->handleRequest($request);

          if ($form->isValid() && $form->isSubmitted()) {
            $user->setForgotPasswordHash(null);
            $user->setForgotPasswordExpiry(null);

            //encode the user's password
            $securityEncoder = $this->get('security.password_encoder');
            $salt = base64_encode(random_bytes(64));
            $encodedPassword = $securityEncoder->encodePassword($user, $user->getPlainPassword());
            $user->setPassword($encodedPassword);

            //now persist the user
            $this->persistEntity($user);

            $this->addFlashMessage('success', 'common.entities.BaseAppUser.flash.passwordChanged');
          } else {
            $templateData = ['form' => $form->createView()];
          }
        }
      }
    }

    //render the form
    return $this->render('BazookasCommonBundle:AppUser\ForgotPassword:passwordReset.html.twig', $templateData);
  }
}
