<?php

namespace Bazookas\CommonBundle\Entity\Interfaces;

use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;

interface AccessControlInterface {

  //These constants should always be the same as the constants in the admin list controller
  const ACTION_EDIT = 'edit';
  const ACTION_ADD = 'add';
  const ACTION_VIEW = 'view';
  const ACTION_REMOVE = 'remove';
  const ACTION_EXPORT = 'export';
  const ACTION_BULK_EDIT = 'bulkEdit';
  const ACTION_BULK_REMOVE = 'bulkRemove';

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canAdd(AuthorizationChecker $auth, BaseCMSUser $user);

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canEdit(AuthorizationChecker $auth, BaseCMSUser $user);

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canDelete(AuthorizationChecker $auth, BaseCMSUser $user);

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canView(AuthorizationChecker $auth, BaseCMSUser $user);

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canExport(AuthorizationChecker $auth, BaseCMSUser $user);

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canBulkEdit(AuthorizationChecker $auth, BaseCMSUser $user);

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @return bool
   */
  public function canBulkRemove(AuthorizationChecker $auth, BaseCMSUser $user);

  /**
   * @param AuthorizationChecker $auth
   * @param BaseCMSUser $user
   * @param string $action
   * @return bool
   */
  public function performAccessCheck(AuthorizationChecker $auth, BaseCMSUser $user, $action);

  /**
   * @return string
   */
  public function getRoleRequiredForAdd();

  /**
   * @return string
   */
  public function getRoleRequiredForEdit();

  /**
   * @return string
   */
  public function getRoleRequiredForDelete();

  /**
   * @return string
   */
  public function getRoleRequiredForView();

  /**
   * @return string
   */
  public function getRoleRequiredForExport();

}
