<?php

namespace Bazookas\CommonBundle\Entity\Traits;

use Doctrine\ORM\Event\PreUpdateEventArgs;
use Doctrine\ORM\Mapping as ORM;

trait TimestampableTrait
{

  /**
   * @ORM\Column(type="datetime", nullable=true)
   * @var \DateTime
   */
  protected $created;

  /**
   * @ORM\Column(type="datetime", nullable=true)
   * @var \DateTime
   */
  protected $modified;

  public function __construct() {
    $this->created = new \DateTime();
    $this->created->setTimezone(new \DateTimeZone('UTC'));
    $this->modified = new \DateTime();
    $this->modified->setTimezone(new \DateTimeZone('UTC'));
  }

  /**
   * @return \DateTime
   */
  public function getCreated() {
    return $this->created;
  }

  /**
   * @param \DateTime|null $created
   * @return $this
   */
  public function setCreated(\DateTime $created = null) {
    $this->created = $created;
    $this->created->setTimezone(new \DateTimeZone('UTC'));
    return $this;
  }

  /**
   * @return \DateTime
   */
  public function getModified() {
    return $this->modified;
  }

  /**
   * @param \DateTime|null $modified
   * @return $this
   */
  public function setModified(\DateTime $modified = null) {
    $this->modified = $modified;
    $this->modified->setTimezone(new \DateTimeZone('UTC'));

    return $this;
  }

  /**
   * @ORM\PreUpdate
   * @param PreUpdateEventArgs $preUpdateEvent
   */
  public function updateModified(PreUpdateEventArgs $preUpdateEvent) {
    if (!empty($preUpdateEvent->getEntityChangeSet())) {
      $this->modified = new \DateTime();
      $this->modified->setTimezone(new \DateTimeZone('UTC'));
    }
  }

  /**
   * A Utility function to determine if 2 dates are equal based on the string representation of the datetimes
   * @param $oldDate
   * @param $newDate
   * @return bool
   */
  protected function compareEqualDates($oldDate, $newDate) {
    //TODO add timezone check?
    if ($oldDate instanceof \DateTime && $newDate instanceof \DateTime) {
      if ($oldDate->format('Y-m-d H:i:s') === $newDate->format('Y-m-d H:i:s')) {
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  }

}
