<?php

namespace Bazookas\CronBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Table
 * @ORM\Entity(
 *   repositoryClass="Bazookas\CronBundle\Repository\CronRepository"
 * )
 * @ORM\HasLifecycleCallbacks
 */
class Cron extends BaseEntity {

  // TODO add validation constraints

  //Interval states
  const INTERVAL_SECONDS = 'seconds';
  const INTERVAL_MINUTES = 'minutes';
  const INTERVAL_HOURS = 'hours';
  const INTERVAL_DAYS = 'days';

  //Cron states
  const STATUS_SCHEDULED = 'scheduled';
  const STATUS_INACTIVE = 'inactive';
  const STATUS_FAILED = 'failed';

  //Execution result states
  const STATUS_SUCCESS = 'success';
  const STATUS_LOCKED = 'locked';

  /**
   * @ORM\Column(type="string")
   * @Assert\NotBlank
   */
  private $command;

  /**
   * @ORM\Column(type="integer")
   * @Assert\NotBlank
   */
  private $executionInterval;

  /**
   * @ORM\Column(type="string")
   * @Assert\Choice({"seconds", "minutes", "hours", "days"})
   */
  private $intervalUnit;

  /**
   * @ORM\Column(type="datetime")
   */
  private $nextRunOn;

  /**
   * @ORM\Column(type="string")
   * @Assert\Choice({"scheduled", "inactive", "failed", "locked"})
   */
  private $status = self::STATUS_SCHEDULED;

  /**
   * @ORM\Column(type="datetime", nullable=true)
   */
  private $lastRunOn;

  /**
   * @ORM\Column(type="string", nullable=true)
   * @Assert\Choice({"success", "failed", "locked"})
   */
  private $lastRunStatus;

  /**
   * @ORM\OneToMany(targetEntity="CronHistory", mappedBy="cron", cascade="all")
   * @ORM\OrderBy({"runStart" = "DESC"})
   */
  private $history;

  /**
   * @ORM\Column(type="integer")
   * @Assert\GreaterThanOrEqual(0)
   */
  private $maxExecutionTime = 300;

  /**
   * @ORM\Column(type="integer")
   * @Assert\GreaterThanOrEqual(0)
   */
  private $maxExecutionsBeforeFail = 10;

  /**
   * @ORM\Column(type="integer")
   * @Assert\GreaterThanOrEqual(0)
   */
  private $failureCount = 0;

  /**
   * @ORM\Column(type="boolean")
   */
  private $removeAfterExecute = false;

  /**
   * @ORM\Column(type="string", columnDefinition="VARCHAR(255) NOT NULL DEFAULT 'bazookas.cron.service.BaseCronService'")
   */
  private $cronService = 'bazookas.cron.service.BaseCronService';

  /**
   * @ORM\Column(type="string", nullable=true)
   */
  private $email;

  /**
   * @ORM\Column(type="array", nullable=true)
   */
  private $emailParameters;

  /**
   * @var bool
   * @ORM\Column(type="boolean")
   */
  private $continueAfterFailCount = false;

  /**
   * @var bool
   * @ORM\Column(type="boolean")
   */
  private $notificationSent = false;

  public function __construct() {
    parent::__construct();
    $this->history = new ArrayCollection();
    $this->nextRunOn = new \DateTime('now', new \DateTimeZone('UTC'));
  }

  public function getCommand()
  {
    return $this->command;
  }

  public function setCommand($command)
  {
    $this->command = $command;

    return $this;
  }

  public function getExecutionInterval()
  {
    return $this->executionInterval;
  }

  public function setExecutionInterval($executionInterval)
  {
    $this->executionInterval = $executionInterval;

    return $this;
  }

  public function getIntervalUnit()
  {
    return $this->intervalUnit;
  }

  public function setIntervalUnit($intervalUnit)
  {
    $this->intervalUnit = $intervalUnit;

    return $this;
  }

  public function getNextRunOn()
  {
    return $this->nextRunOn;
  }

  public function setNextRunOn($nextRunOn)
  {
    $this->nextRunOn = $nextRunOn;

    return $this;
  }

  public function getStatus()
  {
    return $this->status;
  }

  public function setStatus($status)
  {
    $this->status = $status;

    return $this;
  }

  public function getLastRunOn()
  {
    return $this->lastRunOn;
  }

  public function setLastRunOn($lastRunOn)
  {
    $this->lastRunOn = $lastRunOn;

    return $this;
  }

  public function getLastRunStatus()
  {
    return $this->lastRunStatus;
  }

  public function setLastRunStatus($lastRunStatus)
  {
    $this->lastRunStatus = $lastRunStatus;

    return $this;
  }

  public function getHistory()
  {
    return $this->history;
  }

  public function setHistory($history)
  {
    $this->history = $history;

    return $this;
  }

  public function addHistory($cronHistory) {
    $this->history->add($cronHistory);
    return $this;
  }

  public function getMaxExecutionTime()
  {
    if (empty($this->maxExecutionTime)) {
      return 300;
    }
    return $this->maxExecutionTime;
  }

  public function setMaxExecutionTime($maxExecutionTime)
  {
    $this->maxExecutionTime = $maxExecutionTime;

    return $this;
  }

  public function getMaxExecutionsBeforeFail()
  {
    return $this->maxExecutionsBeforeFail;
  }

  public function setMaxExecutionsBeforeFail($maxExecutionsBeforeFail)
  {
    $this->maxExecutionsBeforeFail = $maxExecutionsBeforeFail;

    return $this;
  }

  public function getFailureCount()
  {
    return $this->failureCount;
  }

  public function setFailureCount($failureCount)
  {
    $this->failureCount = $failureCount;

    return $this;
  }

  public function incrementFailCount($amount) {
    $this->failureCount += $amount;
    return $this;
  }

  public function getRemoveAfterExecute() {
    return $this->removeAfterExecute;
  }

  public function setRemoveAfterExecute($removeAfterExecute) {
    $this->removeAfterExecute = $removeAfterExecute;
    return $this;
  }

  public function getCronService() {
    return $this->cronService;
  }

  public function setCronService($cronService) {
    $this->cronService = $cronService;
    return $this;
  }

  public function getEmail() {
    return $this->email;
  }

  public function setEmail($email) {
    $this->email = $email;
    return $this;
  }

  public function getEmailParameters() {
    return $this->emailParameters;
  }

  public function setEmailParameters($emailParameters) {
    $this->emailParameters = $emailParameters;
    return $this;
  }

  public function isContinueAfterFailCount()
  {
    return $this->continueAfterFailCount;
  }

  public function setContinueAfterFailCount($continueAfterFailCount)
  {
    $this->continueAfterFailCount = $continueAfterFailCount;
    return $this;
  }

  public function isNotificationSent()
  {
    return $this->notificationSent;
  }

  public function setNotificationSent($notificationSent)
  {
    $this->notificationSent = $notificationSent;
    return $this;
  }

  public function getRoleRequiredForView() {
    return 'ROLE_CRON_ADMIN';
  }

}
