## Using the export-service

There are two ways that you can use the export-service in your project:
- By using the export-action implemented in the ListBaseController
- By calling the export-service directly

### Using the export-action

By default, the export-action is disabled in the ListBaseController. It can be activated using the following code in a controller that extends the ListBaseController:

```php
protected function checkAccess($_action, $_id) {
    ...
    $this->config->setAccess(self::ACTION_EXPORT, true);
    ...
}

public function exportAction(Request $request) {
    $this->exportConfig = [
        ...
    ];
    
    return parent::exportAction($request);
}
```
In the checkAccess-function you allow access to the export-function, while in the exportAction-function you configure the export.
The export-configuration has been left blank in this example, but will be further discussed here:
* [How to set the export-configuration](exportConfig.md)

### Calling the export-function directly

If for some reason you need the export-service, but the class in question does not extend the ListBaseController, there is another way to create exports.

In your code, you will need to create an export-configuration, just like when you use the ListBaseController. Then you call the export-service directly.

Example:
```php
...
# Using another blank export-config, because seriously, just check the other documentation
$exportConfig = [
    ...
];
$exportService = $this->get('bazookas.export.service.ExportService');
$response = $exportService->export($exportConfig);
return $response;
```
