<?php

namespace Bazookas\MediaBundle\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Bazookas\AdminBundle\Controller\ListBaseController;
use Bazookas\AdminBundle\Util\ListConfigurator;
use Bazookas\AdminBundle\Event\AdminListEvent;
use Bazookas\MediaBundle\Form\MediaFolderAdminType;
use Symfony\Component\HttpFoundation\Request;
use Bazookas\MediaBundle\Exception\LinkedMediaEntityException;
use Bazookas\MediaBundle\Entity\MediaFolder;

class MediaFolderAdminController extends ListBaseController
{

  function __construct() {
    // make sure parent construct is called!
    parent::__construct();

    $this->config->entity = MediaFolder::class;
    $this->config->entityName = 'media.entities.mediaFolder.name';
    // $this->config->addField('property', 'table-header', 'sortable', 'td class', 'template');
    $this->config->addField('name', 'media.entities.mediaFolder.fields.name');
    $this->config->addField('parent', 'media.entities.mediaFolder.fields.parent', false, 'text-center', 'BazookasMediaBundle:List\Fields:_parentField.html.twig');

    $this->config->setForm('Bazookas\MediaBundle\Form\MediaFolderAdminType');
  }

  protected function checkAccess($_action, $_id) {
    // when returning false on any action, there is no real need to set these
    // these are only usefull for showing/hiding the buttons
    // $this->config->setAccess(self::ACTION_ADD, false);
    // $this->config->setAccess(self::ACTION_EDIT, false);
    // $this->config->setAccess(self::ACTION_REMOVE, false);
    return $this->isGranted('ROLE_MEDIA_ADMIN');
  }

  public function getConfig() {
    return $this->config;
  }

  //copied from MediaBaseController, maybe find a way to reuse it?
  public function removeAction(Request $request) {
    //Execute access controle check to make sure the user is allowed to perform the action
    $entity = $this->getRepository()->findOneById($this->id);
    $auth = $this->get('security.authorization_checker');
    if (empty($entity) || !$entity->canDelete($auth, $this->getUser())) {
      return $this->accessDeniedRedirect($request);
    }

    $translator = $this->get('translator');
    try {
      $this->removeEntity($this->id);

      // set flash message
      $this->addFlash(
        'info',
        $translator->trans('admin.list.messages.removed', array(
          '%entity%' => $this->getEntityName()
        ))
      );
    } catch(LinkedMediaEntityException $e) {
      $linkedTo = [];
      foreach($e->getPayload() as $entityName => $items) {
        foreach($items as $item) {
          $message = $translator->trans('media.list.messages.couldNotBeRemoved', array(
            '%entity%' => $this->getEntityName()
          ));
          $linkedTo[$message][] = $item->toHumanReadable();
        }
      }

      $this->addFlash(
        'error', $linkedTo
      );

    }

    $route = $request->get('_route');
    return $this->redirectToRoute($route, array(
      '_locale' => $request->getLocale()
    ));
  }

  protected function addListeners($dispatcher) {
    $dispatcher->addListener(AdminListEvent::BEFORE_LIST_QUERY, array($this, 'beforeListQuery' ));
    $dispatcher->addListener(AdminListEvent::AFTER_LIST_QUERY, array($this, 'addSuperAdminOnlyFields' ));
  }

  public function beforeListQuery($event) {
    if (!$this->isGranted('ROLE_SUPER_ADMIN')) {
      $qb = $event->queryBuilder;
      $qb
        ->andWhere($qb->expr()->neq('e.type', ':folderType' ))
        ->setParameter('folderType', MediaFolder::TYPE_HIDDEN_FOLDER)
      ;
    }
  }

  public function addSuperAdminOnlyFields() {
    if ($this->isGranted('ROLE_SUPER_ADMIN')) {
      $this->config->addField('type', 'media.entities.mediaFolder.fields.type');
      $this->config->addField('defaultFolder', 'media.entities.mediaFolder.fields.defaultFolder');
    }
  }
}
