<?php

namespace Bazookas\MediaBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Bazookas\MediaBundle\Entity\Base\BaseMedia;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
* @ORM\Entity
* @ORM\HasLifecycleCallbacks
*/
class Image extends BaseMedia {

  /**
  * @ORM\Column(type="string", nullable=true)
  */
  protected $aspectRatio;

  /**
  * @Assert\File(maxSize="20M",
  * mimeTypes = {
  *    "image/png",
  *    "image/jpeg",
  *    "image/jpg"
  *  },
  *  maxSizeMessage = "media.error.fileSize",
  *  mimeTypesMessage = "media.error.invalidType"
  * )
  */
  protected $file;

  //as width:height
  public function getAspectRatio() {
    return $this->aspectRatio;
  }

  public function setAspectRatio($ratio) {
    $this->aspectRatio = $ratio;
    return $this;
  }

  public function getDirectory() {
    return 'images';
  }

  /**
   * @Assert\Callback
   */
  public function validate(ExecutionContextInterface $context, $payload)
  {
    if (empty($this->file) && empty($this->path)) {
      $context
        ->buildViolation('media.entities.image.fields.file')
        ->atPath('file')
        ->addViolation()
      ;
    }
  }
}
