<?php

namespace Bazookas\MediaBundle\Imagine\Filter\Loader;

use Imagine\Filter\Basic\Resize;
use Imagine\Image\Box;
use Imagine\Image\ImageInterface;
use Liip\ImagineBundle\Imagine\Filter\Loader\LoaderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

class PowerOfTwoFilter implements LoaderInterface {

  /**
   * @param array $options
   * @return array
   */
  private function validateOptions(array $options) {
    $optionsResolver = new OptionsResolver();
    $optionsResolver
      ->setDefaults([
        'maxWidth' => 1024,
        'maxHeight' => 1024
      ])
      ->setAllowedValues('maxWidth', function($value) {
        return ($value & ($value - 1)) == 0;
      })
      ->setAllowedValues('maxHeight', function($value) {
        return ($value & ($value - 1)) == 0;
      })
    ;
    return $optionsResolver->resolve($options);
  }

  public function load(ImageInterface $image, array $options = []) {
    $options = $this->validateOptions($options);

    // get the original image size
    $size = $image->getSize();

    list($newWidth, $newHeight) = $this->calculateDimensions($size->getWidth(), $size->getHeight(), $options['maxWidth'], $options['maxHeight']);

    $newSize = new Box($newWidth, $newHeight);
    $resize = new Resize($newSize);

    // apply resize to image
    $image = $resize->apply($image);
    return $image;
  }

  /**
   * Calculate the previous power of 2
   * @param $number
   * @return int
   */
  private function calculatePrevPower($number)
  {
    // check if size is already a power of two
    if (($number & ($number - 1)) == 0) {
      return $number;
    } elseif ($number < 2) {
      return 1;
    } else {
      for ($i = 0; $number > 1; $i++) {
        $number = $number >> 1;
      }

      return 1 << $i;
    }
  }

  /**
   * Calculate the new dimensions of the image
   * @param $width
   * @param $height
   * @param $maxWidth
   * @param $maxHeight
   * @return array
   */
  private function calculateDimensions($width, $height, $maxWidth, $maxHeight) {
    $newWidth = $this->calculatePrevPower($width);
    if ($newWidth > $maxWidth) {
      $newWidth = $maxWidth;
    }

    $newHeight = $this->calculatePrevPower($height);
    if ($newHeight > $maxHeight) {
      $newHeight = $maxHeight;
    }

    return [$newWidth, $newHeight];
  }

}
